%==========================================================================
%  Function: nlsim()
%     performs a forward simulation of the system that is described by the
%     given ODE dx/dt = f(x,u) and given output function y=g(x, u). The
%     simulation starts at t=0 at the given initial state x_0. Then, one
%     simulation step of length delta_t is performed for each contrtol
%     input in the provided control input trajectory U. It is assumed that
%     a control input is constant during the time interval it is applied. 
%   
%  Synopsis:
%      X, Y, T = nlsim(my_ode, x_0, U, delta_t, my_outFunc)
%   
%  Input Parameter:
%     - my_ode: ODE of the form dx(t)/dt = f(x(t),u(t)) that has to be 
%          integrated
%          <function handle to function f. The inputs and outputs of
%          function f must be column vectors>
%     - x_0: initial state of the system at t=0 <column vector>
%     - U: sequence of control inputs that are applied to the system one
%          after another. Each control input is applied with duration of
%          delta_t, where it is assumed that the control input is constant
%          during that time 
%          <matrix where every columns represent a control input. The
%          column number corresponds to the number of the integration
%          interval when the control input is applied>
%     - delta_t: length of a single integration interval. The total
%                simulated time is equal to delta_t * size(U, 2)
%                <positive float>
%     - my_outFunc: output function of the form y(t) = g(x(t), u(t)) that
%                   maps the state and control input to the output
%                   <function handle to function g. The inputs and outputs
%                   of function g must be column vectors>
%     
%  Output Parameter:
%     - X: Trajectory of system states
%          <matrix where every column corresponds to the (multidimensional) 
%          state of the system at the beginning of a certain time step. The
%          time step is given by the index of that column>
%     - Y: Trajectory of system outputs
%          <matrix where every column corresponds to the (multidimensional) 
%          output of the system at the beginning of a certain time step. 
%          The time step is given by the index of that column>
%     - T: Time line that corresponds to X and Y <row vector>
% =========================================================================
function [X, Y, T] = nlsim(my_ode, x_0, U, delta_t, my_outFunc)
  x = x_0;
  X = zeros(size(x_0,1), size(U,2)+1);
  Y = zeros( size(my_outFunc(x_0), 1), size(U,2)+1);
  X(:,1) = x_0;
  Y(:,1) = my_outFunc(x_0, U(:, 1));
  for step = 1 : size(U, 2)
      x = rk(my_ode, delta_t, x, U(:, step));
      X(:, step+1) = x;
      Y(:, step+1) = my_outFunc(x, U(:, step));
  end
  T = 0:delta_t:delta_t*size(U, 2);
end